@@ -21,6 +21,7 @@ import pyspng
 import simplejpeg
 import compresso
 import fastremap
+import zfpc
 
 from PIL import Image
 
@@ -49,13 +50,14 @@ except ImportError:
 SUPPORTED_ENCODINGS = (
   "raw", "kempressed", "fpzip",
   "compressed_segmentation", "compresso",
-  "jpeg", "png"
+  "jpeg", "png", "zfpc"
 )
 
 def encode(
   img_chunk:np.ndarray, 
   encoding:str, 
-  block_size:Optional[Sequence[int]] = None
+  block_size:Optional[Sequence[int]] = None,
+  compression_params:dict = {},
 ) -> bytes:
   if encoding == "raw":
     return encode_raw(img_chunk)
@@ -64,6 +66,8 @@ def encode(
   elif encoding == "fpzip":
     img_chunk = np.asfortranarray(img_chunk)
     return fpzip.compress(img_chunk, order='F')
+  elif encoding == "zfpc":
+    return zfpc.compress(np.asfortranarray(img_chunk), **compression_params)
   elif encoding == "compressed_segmentation":
     return encode_compressed_segmentation(img_chunk, block_size=block_size)
   elif encoding == "compresso":
@@ -100,6 +104,8 @@ def decode(
     return decode_kempressed(filedata)
   elif encoding == "fpzip":
     return fpzip.decompress(filedata, order='F')
+  elif encoding == "zfpc":
+    return zfpc.decompress(filedata)
   elif encoding == "compressed_segmentation":
     return decode_compressed_segmentation(filedata, shape=shape, dtype=dtype, block_size=block_size)
   elif encoding == "compresso":
@@ -325,9 +331,3 @@ def read_voxel(
     img = decode(filedata, encoding, shape, dtype, block_size, background_color)
     return img[tuple(xyz)][:, np.newaxis, np.newaxis, np.newaxis]
 
-
-
-
-
-
-
