@@ -18,6 +18,7 @@ from tqdm import tqdm
 from cloudfiles import CloudFiles, compression
 
 from cloudvolume import lib, exceptions
+from cloudvolume.lru import LRU
 from ....lib import Bbox, Vec, sip, first, BboxLikeType
 from .... import sharedmemory, chunks
 
@@ -29,12 +30,14 @@ from . import tx, rx
 class PrecomputedImageSource(ImageSourceInterface):
   def __init__(
     self, config, meta, cache,
-    autocrop=False, bounded=True,
-    non_aligned_writes=False,
-    fill_missing=False, 
-    delete_black_uploads=False,
-    background_color=0,
-    readonly=False,
+    autocrop:bool = False, 
+    bounded:bool = True,
+    non_aligned_writes:bool = False,
+    fill_missing:bool = False, 
+    delete_black_uploads:bool = False,
+    background_color:int = 0,
+    readonly:bool = False,
+    lru_bytes:int = 0,
   ):
     self.config = config
     self.meta = meta 
@@ -51,6 +54,8 @@ class PrecomputedImageSource(ImageSourceInterface):
 
     self.shared_memory_id = self.generate_shared_memory_location()
 
+    self.lru = LRU(lru_bytes, size_in_bytes=True)
+
   def generate_shared_memory_location(self):
     return 'precomputed-shm-' + str(uuid.uuid4())
 
@@ -144,13 +149,13 @@ class PrecomputedImageSource(ImageSourceInterface):
 
     if self.is_sharded(mip):
       if renumber:
-        raise ValueError("renumber is only supported for non-shared volumes.")
+        raise ValueError("renumber is only supported for non-sharded volumes.")
 
       scale = self.meta.scale(mip)
       spec = sharding.ShardingSpecification.from_dict(scale['sharding'])
       return rx.download_sharded(
         bbox, mip, 
-        self.meta, self.cache, spec,
+        self.meta, self.cache, self.lru, spec,
         compress=self.config.compress,
         progress=self.config.progress,
         fill_missing=self.fill_missing,
@@ -162,6 +167,7 @@ class PrecomputedImageSource(ImageSourceInterface):
         bbox, mip, 
         meta=self.meta,
         cache=self.cache,
+        lru=self.lru,
         parallel=parallel,
         location=location,
         retain=retain,
@@ -177,6 +183,58 @@ class PrecomputedImageSource(ImageSourceInterface):
         background_color=int(self.background_color),
       )
 
+  def download_files(
+    self, bbox:Bbox, mip:int, 
+    decompress:bool = True, 
+    parallel:int = 1, 
+    cache_only:bool = False
+  ):
+    """
+    Download the files that comprise a cutout image from the dataset
+    without rendering them into an image. 
+
+    bbox: a Bbox object describing what region to download
+    mip: which resolution to fetch, 0 is the highest resolution
+    parallel: how many processes to use for downloading 
+    cache_only: write downloaded files to cache and discard
+      the result to save memory
+
+    Returns: 
+      If sharded:
+        { morton_code: binary }
+      else:
+        { path: binary }
+    """
+    if self.autocrop:
+      bbox = Bbox.intersection(bbox, self.meta.bounds(mip))
+
+    self.check_bounded(bbox, mip)
+
+    if self.is_sharded(mip):
+      scale = self.meta.scale(mip)
+      spec = sharding.ShardingSpecification.from_dict(scale['sharding'])
+      return rx.download_raw_sharded(
+        bbox, mip, 
+        self.meta, self.cache, spec,
+        decompress=decompress,
+        progress=self.config.progress,
+      )
+    else:
+      return rx.download_raw_unsharded(
+        bbox, mip,
+        meta=self.meta,
+        cache=self.cache,
+        decompress=decompress,
+        progress=self.config.progress,
+        parallel=parallel, 
+        green=self.config.green,
+        secrets=self.config.secrets,
+        fill_missing=self.fill_missing,
+        compress_type=self.config.compress,
+        background_color=int(self.background_color),
+        cache_only=cache_only,
+      )
+
   def unique(self, bbox:BboxLikeType, mip:int) -> set:
     """Extract unique values in an efficient way."""
     bbox = Bbox.create(bbox, context=self.meta.bounds(mip))
@@ -191,7 +249,7 @@ class PrecomputedImageSource(ImageSourceInterface):
       spec = sharding.ShardingSpecification.from_dict(scale['sharding'])
       return rx.unique_sharded(
         bbox, mip, 
-        self.meta, self.cache, spec,
+        self.meta, self.cache, self.lru, spec,
         compress=self.config.compress,
         progress=self.config.progress,
         fill_missing=self.fill_missing,
@@ -202,6 +260,7 @@ class PrecomputedImageSource(ImageSourceInterface):
         bbox, mip, 
         meta=self.meta,
         cache=self.cache,
+        lru=self.lru,
         parallel=1,
         fill_missing=self.fill_missing,
         progress=self.config.progress,
@@ -250,8 +309,8 @@ class PrecomputedImageSource(ImageSourceInterface):
       return
 
     return tx.upload(
-      self.meta, self.cache,
-      image, offset, mip,
+      self.meta, self.cache, self.lru,
+      image, offset, mip, 
       compress=self.config.compress,
       compress_level=self.config.compress_level,
       cdn_cache=self.config.cdn_cache,
@@ -285,11 +344,18 @@ class PrecomputedImageSource(ImageSourceInterface):
       protocol=self.meta.path.protocol
     )
 
-    return CloudFiles(
+    exists = CloudFiles(
       self.meta.cloudpath, progress=self.config.progress,
       secrets=self.config.secrets
     ).exists(cloudpaths)
 
+    if len(self.lru):
+      for k,v in exists.items():
+        if v == False:
+          self.lru.pop(k, None)
+
+    return exists
+
   @readonlyguard
   def delete(self, bbox, mip=None):
     if mip is None:
@@ -323,6 +389,10 @@ class PrecomputedImageSource(ImageSourceInterface):
     CloudFiles(self.meta.cloudpath, progress=self.config.progress, secrets=self.config.secrets) \
       .delete(cloudpaths())
 
+    if len(self.lru) > 0:
+      for path in cloudpaths():
+        self.lru.pop(path, None)
+
     if self.cache.enabled:
       CloudFiles('file://' + self.cache.path, progress=self.config.progress, secrets=self.config.secrets) \
         .delete(cloudpaths())
@@ -427,7 +497,7 @@ class PrecomputedImageSource(ImageSourceInterface):
       for srcpaths in sip(cloudpaths, step):
         files = check(cfsrc.get(srcpaths, raw=True))
         cfdest.puts(
-          compression.transcode(files, encoding=compress, level=compress_level), 
+          compression.transcode(files, encoding=compress, level=compress_level, in_place=True), 
           compress=compress,
           content_type=tx.content_type(destvol),
           raw=True
@@ -505,7 +575,11 @@ class PrecomputedImageSource(ImageSourceInterface):
       cutout_bbx -= pt_anchor
 
       chunk = img[ cutout_bbx.to_slices() ]
-      labels[morton_code] = chunks.encode(chunk, self.meta.encoding(mip))
+      labels[morton_code] = chunks.encode(
+        chunk, self.meta.encoding(mip),
+        block_size=self.meta.compressed_segmentation_block_size(mip),
+        compression_params=self.meta.compression_params(mip),
+      )
 
     shard_filename = reader.get_filename(first(labels.keys()))
 
